#ifndef KS_KEYMAP_H
#define KS_KEYMAP_H

static const int NUM_KEYS = 3*16 + 2;
static const int LEFT_SHIFT = 46;
static const int RIGHT_SHIFT = 1;
static const int META = 47;

static const int SHIFT_FLAG = 0x100;
static const int ALTGR_FLAG = 0x200;

// See http://www.usb.org/developers/hidpage/Hut1_12v2.pdf page 53

const int KEY_MAP_LOWER[NUM_KEYS] = {
  0x2a, // 0: Backspace
  0x00, // 1: Right shift
  0x2e, // 2: '
  0x34, // 3: ä
  0x2f, // 4: ü
  0x38, // 5: -
  0x2d, // 6: ß
  0x33, // 7: ö
  0x13, // 8: p
  0x37, // 9: .
  0x57, // 10: +
  0x0f, // 11: l
  0x12, // 12: o
  0x36, // 13: ,
  0x26, // 14: 9
  0x0e, // 15: k
  0x0c, // 16: i
  0x10, // 17: m
  0x25, // 18: 8
  0x0d, // 19: j
  0x18, // 20: u
  0x11, // 21: n
  0x24, // 22: 7
  0x0b, // 23: h
  0x1c, // 24: z
  0x05, // 25: b
  0x23, // 26: 6
  0x0a, // 27: g
  0x17, // 28: t
  0x19, // 29: v
  0x22, // 30: 5
  0x09, // 31: f
  0x15, // 32: r
  0x06, // 33: c
  0x21, // 34: 4
  0x07, // 35: d
  0x08, // 36: e
  0x1b, // 37: x
  0x20, // 38: 3
  0x16, // 39: s
  0x1a, // 40: w
  0x1d, // 41: y
  0x1f, // 42: 2
  0x04, // 43: a
  0x14, // 44: q
  0x1e, // 45: 1
  0x00, // 46: left shift
  0x00, // 47: meta
  0x2c, // 48: space
  0x28, // 49: enter
};

const int KEY_MAP_UPPER[NUM_KEYS] = {
  0x2a, // 0: Backspace
  0x00, // 1: Right shift
  0x2e | SHIFT_FLAG, // 2: `
  0x34 | SHIFT_FLAG, // 3: Ä
  0x2f | SHIFT_FLAG, // 4: Ü
  0x31 | SHIFT_FLAG, // 5: '
  0x37 | SHIFT_FLAG, // 6: :
  0x33 | SHIFT_FLAG, // 7: Ö
  0x13 | SHIFT_FLAG, // 8: P
  0x1e | SHIFT_FLAG, // 9: !
  0x24 | SHIFT_FLAG, // 10: /
  0x0f | SHIFT_FLAG, // 11: L
  0x12 | SHIFT_FLAG, // 12: O
  0x2d | SHIFT_FLAG, // 13: ?
  0x20 | SHIFT_FLAG, // 14: §
  0x0e | SHIFT_FLAG, // 15: K
  0x0c | SHIFT_FLAG, // 16: I
  0x10 | SHIFT_FLAG, // 17: M
  0x38 | SHIFT_FLAG, // 18: _
  0x0d | SHIFT_FLAG, // 19: J
  0x18 | SHIFT_FLAG, // 20: U
  0x11 | SHIFT_FLAG, // 21: N
  0x26 | SHIFT_FLAG, // 22: )
  0x0b | SHIFT_FLAG, // 23: H
  0x1c | SHIFT_FLAG, // 24: Z
  0x05 | SHIFT_FLAG, // 25: B
  0x25 | SHIFT_FLAG, // 26: (
  0x0a | SHIFT_FLAG, // 27: G
  0x17 | SHIFT_FLAG, // 28: T
  0x19 | SHIFT_FLAG, // 29: V
  0x23 | SHIFT_FLAG, // 30: %
  0x09 | SHIFT_FLAG, // 31: F
  0x15 | SHIFT_FLAG, // 32: R
  0x06 | SHIFT_FLAG, // 33: C
  0x22 | SHIFT_FLAG, // 34: %
  0x07 | SHIFT_FLAG, // 35: D
  0x08 | SHIFT_FLAG, // 36: E
  0x1b | SHIFT_FLAG, // 37: X
  0x27 | SHIFT_FLAG, // 38: =
  0x16 | SHIFT_FLAG, // 39: S
  0x1a | SHIFT_FLAG, // 40: W
  0x1d | SHIFT_FLAG, // 41: Y
  0x1f | SHIFT_FLAG, // 42: "
  0x04 | SHIFT_FLAG, // 43: A
  0x14 | SHIFT_FLAG, // 44: Q
  0x36 | SHIFT_FLAG, // 45: ;
  0x00, // 46: left shift
  0x00, // 47: meta
  0x2c, // 48: space
  0x28, // 49: enter
};

const int KEY_MAP_META[NUM_KEYS] = {
  0x2a, // 0: Backspace
  0x00, // 1: Right shift
  0x2e, // 2: '
  0x34, // 3: ä
  0x2f, // 4: ü
  0x64 | ALTGR_FLAG, // 5: |
  0x2d | ALTGR_FLAG, // 6: ß
  0x33, // 7: ö
  0x13, // 8: p
  0x64 | SHIFT_FLAG, // 9: >
  0x27, // 10: 0
  0x4f, // 11: cursor right
  0x12, // 12: o
  0x64, // 13: <
  0x27 | ALTGR_FLAG, // 14: }
  0x51, // 15: cursor down
  0x52, // 16: cursor up
  0x10, // 17: m
  0x24 | ALTGR_FLAG, // 18: {
  0x50, // 19: cursor left
  0x18, // 20: u
  0x11, // 21: n
  0x26 | ALTGR_FLAG, // 22: ]
  0x0b, // 23: h
  0x1c, // 24: z
  0x05, // 25: b
  0x25 | ALTGR_FLAG, // 26: [
  0x0a, // 27: g
  0x17, // 28: t
  0x19, // 29: v
  0x30 | ALTGR_FLAG, // 30: ~
  0x09, // 31: f
  0x15, // 32: r
  0x06, // 33: c
  0x30 | SHIFT_FLAG, // 34: *
  0x07, // 35: d
  0x08, // 36: e
  0x1b, // 37: x
  0x21 | SHIFT_FLAG, // 38: $
  0x16, // 39: s
  0x1a, // 40: w
  0x1d, // 41: y
  0x31, // 42: #
  0x04, // 43: a
  0x14, // 44: q
  0x29, // 45: ESC
  0x00, // 46: left shift
  0x00, // 47: meta
  0x2b, // 48: tab
  0x28, // 49: enter
};

#endif
